from HardenedSteel.globals                                      \
    import get_system_random


from HardenedSteel.facades.integers.random_number_generation    \
    import                                                      \
    generate_signed_integer,                                    \
    generate_unsigned_integer



from HardenedSteel.globals.singletons.SingletonSystenRandom     \
    import                                                      \
    get_system_random,                                          \
    set_system_random



from HardenedSteel.globals.singletons               \
    import                                          \
    get_system_random,                              \
    set_system_random

from HardenedSteel.globals.vars                     \
    import                                          \
    get_maximum_size_of_integer,                    \
    get_zero,                                       \
    get_one,                                        \
    get_minimum_size_of_integer,                    \
    get_maximum_size_of_unsigned_integer



from HardenedSteel.facades.texts.random_text_generation     \
    import                                                  \
    generate_lowercase_label_by_size,                       \
    generate_uppercase_label_by_size,                       \
    generate_label_by_size




from HardenedSteel.globals                          \
    import                                          \
    get_minimum_size_of_integer,                    \
    get_maximum_size_of_integer,                    \
    get_zero,                                       \
    get_maximum_size_of_unsigned_integer,           \
    get_system_random


def generate_unsigned_integer(
        begin: int = get_zero(),
        end: int = get_maximum_size_of_unsigned_integer()
) -> int:
    return get_system_random().randint(
        begin,
        end
    )


def generate_signed_integer(
        begin: int = get_minimum_size_of_integer(),
        end: int = get_maximum_size_of_integer()
) -> int:
    return get_system_random().randint(
        begin,
        end
    )



from HardenedSteel.globals                                      \
    import                                                      \
    get_system_random,                                          \
    get_one

from HardenedSteel.facades.booleans.random_boolean_generation   \
    import generate_random_boolean



from HardenedSteel.facades.texts.characters                                 \
    import                                                                  \
    random_lowercase_letter,                                                \
    random_uppercase_letter,                                                \
    random_letter

from HardenedSteel.facades.texts.exceptions                                 \
    import raise_out_of_boundary_exception

from HardenedSteel.globals                                                  \
    import                                                                  \
    get_maximum_size_of_integer,                                            \
    get_zero


def detect_outside_boundary_exception(
        index: int
):
    raise_out_of_boundary_exception(
        index,
        get_zero(),
        get_maximum_size_of_integer()
    )


def generate_lowercase_label_by_size(
        length_of_label: int
) -> str:
    detect_outside_boundary_exception(
        length_of_label
    )
    r_value: str = ''

    for position                                                            \
            in range(length_of_label):

        r_value = r_value + random_lowercase_letter()

    return r_value


def generate_uppercase_label_by_size(
        length_of_label: int
) -> str:
    detect_outside_boundary_exception(
        length_of_label
    )
    r_value: str = ''

    for position                                                            \
            in range(length_of_label):

        r_value = r_value + random_uppercase_letter()

    return r_value


def generate_label_by_size(
        length_of_label: int
) -> str:
    detect_outside_boundary_exception(
        length_of_label
    )
    r_value: str = ''

    for position                                                            \
            in range(length_of_label):
        r_value = r_value + random_letter()

    return r_value

















from HardenedSteel.globals.vars \
    import maxsize

zero: int = 0
one: int = 1

unsigned_max_size: int = 4294967295

integer_max_size: int = maxsize
integer_min_size: int = -maxsize - one


def get_maximum_size_of_unsigned_integer() -> int:
    global unsigned_max_size
    return unsigned_max_size


def get_minimum_size_of_integer() -> int:
    global integer_min_size
    return integer_min_size


def get_maximum_size_of_integer() -> int:
    global integer_max_size
    return integer_max_size


def get_zero() -> int:
    global zero
    return zero


def get_one() -> int:
    global one
    return one



from sys                                            \
    import maxsize

from HardenedSteel.globals.vars.integer             \
    import                                          \
    get_maximum_size_of_integer,                    \
    get_zero,                                       \
    get_minimum_size_of_integer,                    \
    get_maximum_size_of_unsigned_integer,           \
    get_one



def raise_out_of_boundary_exception(
        index: int,
        minimum: int,
        maximum: int
) -> None:
    if not (
        (minimum <= index)
        and
        (index <= maximum)
    ):
        raise Exception(
            'out of boundary exception - the given value is outside the functions boundary'
        )



from HardenedSteel.facades                      \
    import                                      \
    generate_lowercase_label_by_size,           \
    generate_uppercase_label_by_size,           \
    generate_label_by_size,                     \
    generate_unsigned_integer,                  \
    generate_signed_integer,                    \
    generate_random_boolean



from HardenedSteel.facades.booleans                 \
    import                                          \
    get_system_random,                              \
    get_one


def generate_random_boolean() -> bool:
    random = get_system_random()
    
    return bool(
        random.getrandbits(
            get_one()
        )
    )



from HardenedSteel.facades.booleans             \
    import generate_random_boolean


from HardenedSteel.facades.texts                \
    import                              \
    generate_lowercase_label_by_size,   \
    generate_uppercase_label_by_size,   \
    generate_label_by_size


from HardenedSteel.facades.integers             \
    import                              \
    generate_unsigned_integer,          \
    generate_signed_integer




from HardenedSteel.globals                              \
    import get_system_random


from HardenedSteel.facades.booleans                     \
    import generate_random_boolean


from HardenedSteel.facades.texts.characters.ranges      \
    import                                              \
    random_lowercase_letter,                            \
    random_uppercase_letter,                            \
    random_letter
